/*******************************************************************************
 * Copyright (c) 2007 VTT Technical Research Centre of Finland and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.tests;

import java.io.File;
import java.util.Map;
import java.util.TreeMap;

import org.simantics.databoard.Accessors;
import org.simantics.databoard.Bindings;
import org.simantics.databoard.Datatypes;
import org.simantics.databoard.accessor.MapAccessor;
import org.simantics.databoard.accessor.file.FileMapAccessor;
import org.simantics.databoard.accessor.impl.MapAccessorIterator;
import org.simantics.databoard.type.MapType;

public class TestMapAccessorIterator {
	
	public static void main(String[] args) throws Exception {
		
		Map<Integer, Integer> myMap = new TreeMap<Integer, Integer>( Bindings.INTEGER );
		for (int i=0; i<100; i++) myMap.put(i, i*10);
		
		MapAccessor ma = Accessors.getAccessor( Bindings.getBinding(TreeMap.class, Integer.class, Integer.class), myMap);
		
		
		MapAccessorIterator<Double, Integer> iter = new MapAccessorIterator<Double, Integer>(
				ma, 
				Bindings.DOUBLE, 0.0, false, 99.0, false,
				Bindings.INTEGER, 
				16
				); 
		
		while (iter.next()) {
			System.out.println("Double: "+iter.keyDouble()+"="+iter.valueDouble());
			System.out.println("Normal: "+iter.key()+"="+iter.value());
		}

		
		FileMapAccessor ma2 = Accessors.createFile( File.createTempFile("delete", ".me"), new MapType(Datatypes.INTEGER, Datatypes.INTEGER) );
		for (int i=0; i<100; i++) ma2.put(Bindings.INTEGER, i, Bindings.INTEGER, i*10);
		ma2.flush();
		
		iter = new MapAccessorIterator<Double, Integer>(
				ma2, 
				Bindings.DOUBLE, 0.0, false, 50.0, true,
				Bindings.INTEGER, 
				16
				); 
		
		while (iter.next()) {
			System.out.println("Double: "+iter.keyDouble()+"="+iter.valueDouble());
			System.out.println("Normal: "+iter.key()+"="+iter.value());
		}
		
	}

}

 