/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import org.simantics.databoard.Units;
import org.simantics.databoard.accessor.reference.ChildReference;
import org.simantics.databoard.annotations.Optional;
import org.simantics.databoard.util.Range;

/**
 * NumberType is comparable primitive type:
 *  ByteType, IntegerType, DoubleType, LongType or FloatType 
 * 
 */
public abstract class NumberType extends Datatype {
    	
	/**
	 * Unit string describes the quentity, magnity and unit of the value.
	 *  
	 * @see Units 
	 */
	public @Optional String unit;
    
	@Optional Range range;
	
	/**
	 * Get the unit type.
	 * 
	 * Unit string describes the quentity, magnity and unit of the value.
	 *
	 * @return the unit
	 * @see Units 
	 */
	public String getUnit() {
		return unit;
	}

	/**
	 * Set the unit type.
	 * Unit string describes the quentity, magnity and unit of the value.
	 * 	
	 * @param unit the unit type
	 * @see Units 
	 */
	public void setUnit(String unit) {
		this.unit = unit;
	}
  
	public Range getRange() {
		return range;
	}
	
	public String getRangeStr() {
		return range==null ? null : range.toString();
	}

	public void setRange(String range) {
    	this.range = range == null ? null : Range.valueOfUnchecked(range);
	}
	
	public void setRange(Range range) {
		this.range = range;
	}
	
    @Override
    public int getComponentCount() {
    	return 0;
    }
    
    @Override
    public Datatype getComponentType(int index) {
    	throw new IllegalArgumentException();
    }
    
    @Override
    public Datatype getComponentType(ChildReference path) {
    	if (path==null) return this;
    	throw new IllegalArgumentException();
    }	
	
}

